/******************************************************************************
 *{@C
 *      Copyright:      2005-2008 Paul Obermeier (obermeier@tcl3d.org)
 *
 *                      See the file "Tcl3D_License.txt" for information on
 *                      usage and redistribution of this file, and for a
 *                      DISCLAIMER OF ALL WARRANTIES.
 *
 *      Module:         Tcl3D -> tcl3dOgl
 *      Filename:       tcl3dOgl.i
 *
 *      Author:         Paul Obermeier
 *
 *      Description:    Main SWIG files for wrapping the OpenGL library, the
 *                      GLU library as well as GLUT compatible basic shapes.
 *                      This file is responsible for the Tcl package tcl3dOgl.
 *
 *                      The macro USE_EXT_IF must be defined at compilation
 *                      time to include extended functionality like
 *                      operating systems specific OpenGL functions
 *                      into the Tcl3D package.
 *
 *****************************************************************************/

// The name of our generated Tcl package.

%module tcl3dOgl


// Code to initialize the Tk stubs library.

%{
#include <tk.h>
%}

%init %{
#ifdef USE_TK_STUBS
    if (Tk_InitStubs(interp, (char*)"8.4", 0) == NULL) {
      return TCL_ERROR;
    }
#endif
%}


// Standard modules from the SWIG library

%include typemaps.i


// Common Tcl3D wrapper modules.

%include tcl3dVectors.i
%include tcl3dConstHash.i
%include tcl3dPointer.i
%include tcl3dArgv.i
%include tcl3dTypemaps.i


// Wrap the GL core library.

%include gl.i


// Wrap the GLUT shapes.

%{
#include "glutShapes.h"
%}

%include glutShapes.h


#ifdef USE_EXT_IF

// Wrap the GLEXT library.

%include glext.i

#if defined (WIN32)
%include wglext.i
#else
// %include glxext.i
#endif

// OPA TODO Why here? Use it in tcl3dutil
%include list2vector.i

#endif // USE_EXT_IF

// Wrap the GLU library.
// Note: This must be at the end of the include list, as this SWIG file
// redefines the standard typemaps for "float *" and "double *".
// All other SWIG files rely on this standard mapping.

%include glu.i
