package require tcl3d

info commands tcl3dMatf*

set x  [tcl3dVector GLfloat 16]
set tfm [tcl3dVector GLfloat 16]
set tv [tcl3dVectorFromArgs GLfloat 1 2 3]
set p  [tcl3dVectorFromArgs GLfloat 1 2 3]
set resp [tcl3dVectorFromArgs GLfloat 10 10 10]
set axis [tcl3dVectorFromArgs GLfloat 1 1 1]
tcl3dVec3fNormalize $axis

puts "New matrix:"
set mi [tcl3dVector GLfloat 16]
tcl3dMatfPrint $mi
puts ""

puts "Identity matrix:"
tcl3dMatfIdentity $mi
tcl3dMatfPrint $mi
puts ""

puts "Copied identity matrix:"
tcl3dMatfCopy $mi $x
tcl3dMatfPrint $x
puts ""

puts "Translation matrix (created with tcl3dMatfTranslate):"
tcl3dMatfTranslate 1 2 3 $x
tcl3dMatfPrint $x
puts ""

puts "Translation matrix (created with tcl3dMatfTranslatev):"
tcl3dMatfCopy $mi $x
tcl3dMatfTranslatev $tv $x
tcl3dMatfPrint $x
puts ""

puts "Rotation matrix (90 degrees around X):"
tcl3dMatfCopy $mi $x
tcl3dMatfRotateX 90 $x
tcl3dMatfPrint $x
puts ""

puts "Rotation matrix (90 degrees around Y):"
tcl3dMatfCopy $mi $x
tcl3dMatfRotateY 90 $x
tcl3dMatfPrint $x
puts ""

puts "Rotation matrix (90 degrees around Z):"
tcl3dMatfCopy $mi $x
tcl3dMatfRotateZ 90 $x
tcl3dMatfPrint $x
puts ""

puts "Rotation matrix (90 degrees around axis (1,1,1)):"
tcl3dMatfCopy $mi $x
tcl3dMatfRotate 90 $axis $x
tcl3dMatfPrint $x
puts ""

puts "Scale matrix (created with tcl3dMatfScale):"
tcl3dMatfCopy $mi $x
tcl3dMatfScale 1 2 3 $x
tcl3dMatfPrint $x
puts ""

puts "Scale matrix (created with tcl3dMatfScalev):"
tcl3dMatfCopy $mi $x
tcl3dMatfScalev $tv $x
tcl3dMatfPrint $x
puts ""

puts "Translation matrix applied to point (1,2,3)):"
tcl3dMatfCopy $mi $tfm
tcl3dMatfTranslate 3 2 1 $tfm
tcl3dMatfTransformPoint $p $tfm $resp
tcl3dMatfPrint $tfm
puts -nonewline "Transformed point: "
tcl3dVec3fPrint $resp
puts ""

puts "Translation matrix applied to vector (1,2,3)):"
tcl3dMatfCopy $mi $tfm
tcl3dMatfTranslate 3 2 1 $tfm
tcl3dMatfTransformVector $p $tfm $resp
tcl3dMatfPrint $tfm
puts -nonewline "Transformed vector: "
tcl3dVec3fPrint $resp
puts ""

exit
